Package: olsrr
Type: Package
Title: Tools for Building OLS Regression Models
Version: 0.7.0
Authors@R: person("Aravind", "Hebbali", email = "hebbali.aravind@gmail.com", role = c("aut", "cre"))
Description: Tools designed to make it easier for users, particularly beginner/intermediate R users 
    to build ordinary least squares regression models. Includes comprehensive regression output, 
    heteroskedasticity tests, collinearity diagnostics, residual diagnostics, measures of influence, 
    model fit assessment and variable selection procedures.
Depends: R(>= 3.3)
Imports: car, ggplot2, goftest, graphics, gridExtra, nortest, stats,
        utils, xplorerr
Suggests: covr, descriptr, knitr, rmarkdown, testthat, vdiffr
License: MIT + file LICENSE
URL: https://olsrr.rsquaredacademy.com/,
        https://github.com/rsquaredacademy/olsrr
BugReports: https://github.com/rsquaredacademy/olsrr/issues
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-21 10:53:56 UTC; HP
Author: Aravind Hebbali [aut, cre]
Maintainer: Aravind Hebbali <hebbali.aravind@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-22 09:00:08 UTC
Built: R 4.6.0; ; 2026-04-25 14:06:40 UTC; unix
