# Generated by roxygen2: do not edit by hand

S3method(ols_anova,default)
S3method(ols_coll_diag,default)
S3method(ols_compare_model_performance,lm)
S3method(ols_correlations,default)
S3method(ols_model_performance,default)
S3method(ols_pure_error_anova,default)
S3method(ols_regress,default)
S3method(ols_regress,lm)
S3method(ols_step_all_possible,default)
S3method(ols_step_backward_adj_r2,default)
S3method(ols_step_backward_aic,default)
S3method(ols_step_backward_p,default)
S3method(ols_step_backward_r2,default)
S3method(ols_step_backward_sbc,default)
S3method(ols_step_backward_sbic,default)
S3method(ols_step_best_subset,default)
S3method(ols_step_both_adj_r2,default)
S3method(ols_step_both_aic,default)
S3method(ols_step_both_p,default)
S3method(ols_step_both_r2,default)
S3method(ols_step_both_sbc,default)
S3method(ols_step_both_sbic,default)
S3method(ols_step_forward_adj_r2,default)
S3method(ols_step_forward_aic,default)
S3method(ols_step_forward_p,default)
S3method(ols_step_forward_r2,default)
S3method(ols_step_forward_sbc,default)
S3method(ols_step_forward_sbic,default)
S3method(ols_test_bartlett,default)
S3method(ols_test_breusch_pagan,default)
S3method(ols_test_f,default)
S3method(ols_test_normality,default)
S3method(ols_test_normality,lm)
S3method(ols_test_score,default)
S3method(plot,ols_step_all_possible)
S3method(plot,ols_step_backward_adj_r2)
S3method(plot,ols_step_backward_aic)
S3method(plot,ols_step_backward_p)
S3method(plot,ols_step_backward_r2)
S3method(plot,ols_step_backward_sbc)
S3method(plot,ols_step_backward_sbic)
S3method(plot,ols_step_best_subset)
S3method(plot,ols_step_both_adj_r2)
S3method(plot,ols_step_both_aic)
S3method(plot,ols_step_both_p)
S3method(plot,ols_step_both_r2)
S3method(plot,ols_step_both_sbc)
S3method(plot,ols_step_both_sbic)
S3method(plot,ols_step_forward_adj_r2)
S3method(plot,ols_step_forward_aic)
S3method(plot,ols_step_forward_p)
S3method(plot,ols_step_forward_r2)
S3method(plot,ols_step_forward_sbc)
S3method(plot,ols_step_forward_sbic)
S3method(print,ols_anova)
S3method(print,ols_coll_diag)
S3method(print,ols_correlations)
S3method(print,ols_model_performance)
S3method(print,ols_pure_error_anova)
S3method(print,ols_regress)
S3method(print,ols_step_all_possible)
S3method(print,ols_step_backward_adj_r2)
S3method(print,ols_step_backward_aic)
S3method(print,ols_step_backward_p)
S3method(print,ols_step_backward_r2)
S3method(print,ols_step_backward_sbc)
S3method(print,ols_step_backward_sbic)
S3method(print,ols_step_best_subset)
S3method(print,ols_step_both_adj_r2)
S3method(print,ols_step_both_aic)
S3method(print,ols_step_both_p)
S3method(print,ols_step_both_r2)
S3method(print,ols_step_both_sbc)
S3method(print,ols_step_both_sbic)
S3method(print,ols_step_forward_adj_r2)
S3method(print,ols_step_forward_aic)
S3method(print,ols_step_forward_p)
S3method(print,ols_step_forward_r2)
S3method(print,ols_step_forward_sbc)
S3method(print,ols_step_forward_sbic)
S3method(print,ols_test_bartlett)
S3method(print,ols_test_breusch_pagan)
S3method(print,ols_test_f)
S3method(print,ols_test_normality)
S3method(print,ols_test_score)
export(ols_aic)
export(ols_anova)
export(ols_apc)
export(ols_coll_diag)
export(ols_compare_model_performance)
export(ols_correlations)
export(ols_count_obs)
export(ols_count_parameters)
export(ols_eigen_cindex)
export(ols_fpe)
export(ols_get_call)
export(ols_get_data)
export(ols_get_deviance)
export(ols_get_df)
export(ols_get_formula)
export(ols_get_interaction_terms)
export(ols_get_intercept)
export(ols_get_model_matrix)
export(ols_get_parameters)
export(ols_get_predicted)
export(ols_get_predictors)
export(ols_get_residuals)
export(ols_get_response)
export(ols_get_sigma)
export(ols_get_variables)
export(ols_get_vcov)
export(ols_hadi)
export(ols_has_intercept)
export(ols_hsp)
export(ols_launch_app)
export(ols_leverage)
export(ols_mallows_cp)
export(ols_model_info)
export(ols_model_performance)
export(ols_msep)
export(ols_plot_added_variable)
export(ols_plot_comp_plus_resid)
export(ols_plot_cooksd_bar)
export(ols_plot_cooksd_chart)
export(ols_plot_dfbetas)
export(ols_plot_dffits)
export(ols_plot_diagnostics)
export(ols_plot_fm)
export(ols_plot_hadi)
export(ols_plot_obs_fit)
export(ols_plot_reg_line)
export(ols_plot_resid_box)
export(ols_plot_resid_fit)
export(ols_plot_resid_fit_spread)
export(ols_plot_resid_hist)
export(ols_plot_resid_lev)
export(ols_plot_resid_pot)
export(ols_plot_resid_qq)
export(ols_plot_resid_regressor)
export(ols_plot_resid_spread)
export(ols_plot_resid_stand)
export(ols_plot_resid_stud)
export(ols_plot_resid_stud_fit)
export(ols_plot_response)
export(ols_pred_rsq)
export(ols_prep_avplot_data)
export(ols_prep_cdplot_data)
export(ols_prep_cdplot_outliers)
export(ols_prep_dfbeta_data)
export(ols_prep_dfbeta_outliers)
export(ols_prep_dsrvf_data)
export(ols_prep_outlier_obs)
export(ols_prep_regress_x)
export(ols_prep_regress_y)
export(ols_prep_rfsplot_fmdata)
export(ols_prep_rfsplot_rsdata)
export(ols_prep_rstudlev_data)
export(ols_prep_rvsrplot_data)
export(ols_prep_srchart_data)
export(ols_prep_srplot_data)
export(ols_press)
export(ols_pure_error_anova)
export(ols_regress)
export(ols_sbc)
export(ols_sbic)
export(ols_step_all_possible)
export(ols_step_all_possible_betas)
export(ols_step_backward_adj_r2)
export(ols_step_backward_aic)
export(ols_step_backward_p)
export(ols_step_backward_r2)
export(ols_step_backward_sbc)
export(ols_step_backward_sbic)
export(ols_step_best_subset)
export(ols_step_both_adj_r2)
export(ols_step_both_aic)
export(ols_step_both_p)
export(ols_step_both_r2)
export(ols_step_both_sbc)
export(ols_step_both_sbic)
export(ols_step_forward_adj_r2)
export(ols_step_forward_aic)
export(ols_step_forward_p)
export(ols_step_forward_r2)
export(ols_step_forward_sbc)
export(ols_step_forward_sbic)
export(ols_test_bartlett)
export(ols_test_breusch_pagan)
export(ols_test_correlation)
export(ols_test_f)
export(ols_test_normality)
export(ols_test_outlier)
export(ols_test_score)
export(ols_vif_tol)
export(rvsr_plot_shiny)
importFrom(car,Anova)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,annotate)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_dotplot)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_label)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_linerange)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_shape_manual)
importFrom(ggplot2,scale_size_manual)
importFrom(ggplot2,stat_qq)
importFrom(ggplot2,stat_smooth)
importFrom(ggplot2,theme)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(goftest,cvm.test)
importFrom(graphics,hist)
importFrom(gridExtra,grid.arrange)
importFrom(gridExtra,marrangeGrob)
importFrom(nortest,ad.test)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,coefficients)
importFrom(stats,complete.cases)
importFrom(stats,confint.lm)
importFrom(stats,cooks.distance)
importFrom(stats,cor)
importFrom(stats,deviance)
importFrom(stats,df.residual)
importFrom(stats,dfbetas)
importFrom(stats,dffits)
importFrom(stats,dnorm)
importFrom(stats,ecdf)
importFrom(stats,fitted)
importFrom(stats,fitted.values)
importFrom(stats,formula)
importFrom(stats,hatvalues)
importFrom(stats,ks.test)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,lsfit)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,rstandard)
importFrom(stats,rstudent)
importFrom(stats,sd)
importFrom(stats,shapiro.test)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(utils,combn)
importFrom(utils,tail)
