# output from model performance is as expected

    Code
      ols_model_performance(model)
    Output
                    Model Performance               
      ---------------------------------------------
      R                 0.909       AIC     158.643 
      R-Squared         0.827       SBC     165.972 
      Adj. R-Squared    0.808       SBIC     68.933 
      Pred R-Squared    0.768       RMSE      2.468 
      ---------------------------------------------
        AIC: Akaike Information Criteria 
        SBC: Schwarz Bayesian Criteria 
       SBIC: Sawa's Bayesian Criteria 
       RMSE: Root Mean Square Error 
      

